	/*
 * Copyright (C) 2014 Mikel Penagarikano
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.gtts.mediaevalquesst2015;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 *
 * @author mpenagar
 */
public class Main {

	public static void main(String[] args) throws Exception {
		double prior = Double.NaN;
		Double defaultScore = null;
		switch (args.length) {
			case 6: defaultScore = Double.valueOf(args[5]);
			case 5: prior = Double.parseDouble(args[4]);
					break;
			default :
					System.err.println("\n  Syntax:  java -jar MediaevalQUESST2014.jar  stdlistFileName rttmFileName ecfFileName tlistFileName prior <defaultScore>\n");
					System.exit(1);
		}
				
		Map<String,Integer> modelMap = TList.load(args[3],false);
		Map<String,Integer> testMap = ECF.load(args[2],false);
		boolean[][] groundTruth = RTTM.load(args[1],modelMap,testMap,false);
		Set<String> nullModels = new HashSet<String>();
		groundTruth = removeUnseenModels(modelMap,groundTruth,nullModels,false);
		STDSystem sys = STDSystem.load(args[0],modelMap,nullModels,testMap,groundTruth,defaultScore,false);

		System.out.println("\nMain Results:\n");
		evalTWV(sys,prior,true);		
		evalCnxe(sys,prior);
		System.out.println("\nModel-dependent calibration Results:\n");
		System.out.print("Upper-bound TWV (model-dependent offset)  -  ");
		evalTWV(Calibration.byModelTWV(sys, prior),prior,false);		
		System.out.print("Lower-bound Cnxe (model-dependent PAV)    -  ");
		evalCnxe(Calibration.byModelPAV(sys),prior);
	}
	
	private static void evalTWV(STDSystem sys, double prior, boolean writePlotFiles){
		double beta = (1.0-prior)/prior;
		TermWeightedValue twv = sys.estimateTWV(beta);
		System.out.print(
				"actTWV: " + (float)twv.actualValue +
				"  maxTWV: " + (float)twv.maxValue() +
				"  Threshold: " + (float)twv.maxValueThreshold() +
		"\n");
		if (writePlotFiles) twv.writePlotFiles(sys.name);	
	}
	
	private static void evalCnxe(STDSystem sys, double prior){
		System.out.print("actCnxe: " + (float)Cnxe.nxe(sys.score,sys.isTarget,prior) + "  ");
		sys.score = PoolAdjacentViolators.calibrate(sys.score,sys.isTarget,null,null);
		System.out.print("minCnxe: " + (float)Cnxe.nxe(sys.score,sys.isTarget,prior) + "\n");		
	}

	
	
	
	private static boolean[][] removeUnseenModels(Map<String,Integer> modelMap,boolean[][] groundTruth,Set<String> nullModels,boolean quiet){
		List<Integer> nullIndex = new ArrayList<Integer>();
		loop: for (int m = 0; m < groundTruth.length; m++) {
			for (boolean isTarget : groundTruth[m]) if (isTarget) continue loop;
			nullIndex.add(m);
		}
		if (!quiet) System.out.print("Removing unseen models metadata ... ");
		for (Map.Entry<String,Integer> e : modelMap.entrySet())
			if (nullIndex.contains(e.getValue())) nullModels.add(e.getKey());
		for (String s : nullModels) modelMap.remove(s);

		// map old indexes to new indexes
		int[] old2new = new int[groundTruth.length];
		for (int i=0,j=0;i<groundTruth.length;i++) old2new[i] = nullIndex.contains(i) ? -1 : j++;
		for (int i = 0; i < old2new.length; i++)
			if (old2new[i] != -1) groundTruth[old2new[i]] = groundTruth[i];
		groundTruth = Arrays.copyOf(groundTruth,groundTruth.length - nullIndex.size());		
		for (Map.Entry<String,Integer> e : modelMap.entrySet()) e.setValue(old2new[e.getValue()]);
		
		if (!quiet) System.out.println(" Done (removed: " + nullIndex.size() + ")");
		return groundTruth;
	}
	
	
}

